from functools import wraps

from .response import HttpResponseUnauthorized
from .basicauthutils import validate_request


def basic_auth_required(func):
    @wraps(func)
    def _wrapped(request, *args, **kwargs):
        if not validate_request(request):
            return HttpResponseUnauthorized()
        return func(request, *args, **kwargs)
    return _wrapped



from .basicauthutils import validate_request
from .compat import MiddlewareMixin
from .response import HttpResponseUnauthorized


class BasicAuthMiddleware(MiddlewareMixin):
    def process_request(self, request):
        if not validate_request(request):
            return HttpResponseUnauthorized()
        return None



import sys

import django

PY2 = sys.version_info[0] == 2
PY3 = sys.version_info[0] == 3

DJ18 = django.VERSION[:2] == (1, 8)

if PY2:
    from urllib import unquote_plus  # NOQA
else:
    from urllib.parse import unquote_plus  # NOQA


if DJ18:
    class MiddlewareMixin(object):
        pass
else:
    from django.utils.deprecation import MiddlewareMixin  # NOQA



from django.conf import settings
from django.http import HttpResponse


class HttpResponseUnauthorized(HttpResponse):
    status_code = 401

    def __init__(self):
        super(HttpResponseUnauthorized, self).__init__(
            """<html><head><title>Basic auth required</title></head>
               <body><h1>Authorization Required</h1></body></html>""",
        )
        realm = getattr(settings, 'BASICAUTH_REALM', 'Secure resource')
        self['WWW-Authenticate'] = 'Basic realm="{}"'.format(realm)



import os
from setuptools import setup

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.rst')).read()
CHANGES = open(os.path.join(here, 'CHANGES.txt')).read()

setup(
    name='django-basicauth',
    version='0.4',
    author='Hiroki KIYOHARA',
    author_email='hirokiky@gmail.com',
    url='https://github.com/hirokiky/django-basicauth/',
    license='MIT',
    description="Basic auth utilities for Django.",
    long_description=README + '\n\n' + CHANGES,
    packages=['basicauth'],
    install_requires=['Django>=1.8,!=1.9,<2.0'],
    include_package_data=True,
    test_suite="tests",
    zip_safe=False,
)



import base64
import binascii

from django.conf import settings

from .compat import unquote_plus


def extract_basicauth(authorization_header, encoding='utf-8'):
    splitted = authorization_header.split(' ')
    if len(splitted) != 2:
        return None

    auth_type, auth_string = splitted

    if 'basic' != auth_type.lower():
        return None

    try:
        b64_decoded = base64.b64decode(auth_string)
    except (TypeError, binascii.Error):
        return None
    try:
        auth_string_decoded = b64_decoded.decode(encoding)
    except UnicodeDecodeError:
        return None

    splitted = auth_string_decoded.split(':')

    if len(splitted) != 2:
        return None

    username, password = map(unquote_plus, splitted)
    return username, password


def validate_request(request):
    """Check an incoming request.

    Returns:
        - True if authentication passed
        - Adding request['REMOTE_USER'] as authenticated username.
    """
    if getattr(settings, 'BASICAUTH_DISABLE', False):
        # Not to use this env
        return True

    if 'HTTP_AUTHORIZATION' not in request.META:
        return False

    authorization_header = request.META['HTTP_AUTHORIZATION']
    ret = extract_basicauth(authorization_header)
    if not ret:
        return False

    username, password = ret

    if settings.BASICAUTH_USERS.get(username) != password:
        return False

    request.META['REMOTE_USER'] = username
    return True
